
class GenericScoreAnnounce expands Mutator config(GenericScoreAnnounce);

// Generic check to prevent multiple instances
var bool bInitialized;

//GSAnnounce
// These are the internal variables that are used to track the scores
// and follow who is leading, etc.
var Pawn Leaders[32], LastLeaders[32], Player[32];
var int  numLeaders, numLastLeaders, numPawns;
var int	fragsLeft;
var int numLastPlayers, numLastSpectators;

// The sounds are stored as string values in the ini file and
// referenced by using these values
var config string OneFragLeft;
var config string TwoFragsLeft;
var config string ThreeFragsLeft;
var config string TakenTheLead;
var config string TiedForLead;
var config string LostTheLead;

// The string values need to be assigned to a proper sound variable so they
// can be called on the client
var Sound OneFragLeftSound;
var Sound TwoFragsLeftSound;
var Sound ThreeFragsLeftSound;
var Sound TakenTheLeadSound;
var Sound TiedForLeadSound;
var Sound LostTheLeadSound;

function PostBeginPlay()
{
	//Helps ensure only one copy runs. 
   if (bInitialized)
		return;
	bInitialized = True;
	
	//Sets the beginning values
   numLastPlayers = Level.Game.NumPlayers;
   numLastSpectators = Level.Game.NumSpectators;

   //Sets up the timer to run
   SetTimer(0.1, True);
   
   //Assigns all the ini declarations to their proper variables
   OneFragLeftSound 		= sound(DynamicLoadObject(OneFragLeft, 	class'Sound'));
	TwoFragsLeftSound 	= sound(DynamicLoadObject(TwoFragsLeft, 	class'Sound'));
	ThreeFragsLeftSound 	= sound(DynamicLoadObject(ThreeFragsLeft, class'Sound'));
	TakenTheLeadSound 	= sound(DynamicLoadObject(TakenTheLead, 	class'Sound'));
	TiedForLeadSound 		= sound(DynamicLoadObject(TiedForLead, 	class'Sound'));
	LostTheLeadSound 		= sound(DynamicLoadObject(LostTheLead, 	class'Sound'));

	//Properly adds itself to the mutator chain
   if (NextMutator != None)
		NextMutator.PostBeginPlay();
}

function Timer()
{
   local Pawn P;

	//Check on super, make sure nothing gets broken
   Super.Timer();

	//A change in playercount causes a reorder to see if the leader is changed
   if ((Level.Game.NumPlayers != numLastPlayers) || (Level.Game.NumSpectators > numLastSpectators))
   {
      announce();
   }
   //Variable update
   numLastPlayers = Level.Game.NumPlayers;
   numLastSpectators = Level.Game.NumSpectators;
}

function ModifyPlayer(Pawn Other)
{
	//New player joins? Better recheck the leader list
   if (!(Level.Game.bTeamGame))
      if (addPawnToList(Other))
	  		announce();
}

function bool isLeader(Pawn P)
{
   local int i;
   for (i = 0; i < numLeaders; i++)
   {
      if (P == Leaders[i])
	 		return True;
   }
   return False;
}

function bool wasLeader(Pawn P)
{
   local int i;
   for (i = 0; i < numLastLeaders; i++)
   {
      if (P == LastLeaders[i])
	 		return True;
   }
   return False;
}

function bool addPawnToList(Pawn P)
{
   local int i;
   for (i = 0; i < numPawns; i++)
   {
      if (Player[i] == P)
	 		return False;
   }
   Player[i] = P;
   numPawns++;
   return True;
}

function announce()
{
   local Pawn P;
   local float maxScore;
   local int i, j;
   local PlayerPawn PP;
   local TournamentGameReplicationInfo TGRI;

   if (Level.Game.bTeamGame)
   	return;

   maxScore = -99999;
   numLeaders = 0;
   PP = None;

   for(P = Level.Pawnlist; P != None; P = P.Nextpawn)
   {
      if ((Bot(P) != None || PlayerPawn(P) != None) && P.PlayerReplicationInfo != None)
      {
         if (P.PlayerReplicationInfo.Score > maxScore && !P.IsA('Spectator'))
         {
	    		maxScore = P.PlayerReplicationInfo.Score;
	    		numLeaders = 1;
	    		Leaders[numLeaders - 1] = P;
         }
         else if (P.PlayerReplicationInfo.Score == maxScore && !P.IsA('Spectator'))
         {
	    		numLeaders++;
	    		Leaders[numLeaders - 1] = P;
         }
         if (PP == None && PlayerPawn(P) != None)
	    		PP = PlayerPawn(P);
      }
   }
   if ((numLeaders == 1) && ((numLastLeaders > 1) || ((numLastLeaders == 1) && !wasLeader(leaders[0]))))
   {
      if (PlayerPawn(Leaders[0]) != None)
         PlayerPawn(Leaders[0]).ClientPlaySound(TakenTheLeadSound, False);

      for (i = 0; i < numLastLeaders; i++)
      {
         if ((PlayerPawn(LastLeaders[i]) != None) && (LastLeaders[i] != Leaders[0]))
            PlayerPawn(LastLeaders[i]).ClientPlaySound(LostTheLeadSound, False);
      }
   }
   else if (numLeaders > 1)
   {
      for (i = 0; i < numLeaders; i++)
      {
         if ((PlayerPawn(Leaders[i]) != None) && (((numLastLeaders == 1) && (wasLeader(Leaders[i]))) || (!wasLeader(Leaders[i]))))
	    		PlayerPawn(Leaders[i]).ClientPlaySound(TiedForLeadSound, False);
      }
      for (i = 0; i < numLastLeaders; i++)
      {
         if (!(isLeader(lastLeaders[i])) && (PlayerPawn(lastLeaders[i]) != None))
	    		PlayerPawn(lastLeaders[i]).ClientPlaySound(LostTheLeadSound, False);
      }
   }
   for (i = 0; i < numLeaders; i++)
   {
      LastLeaders[i] = Leaders[i];
   }
   numLastLeaders = numLeaders;
   if ((PP != None) && (Level.Game.GameName != "Last Man Standing"))
   {
      TGRI = TournamentGameReplicationInfo(PP.GameReplicationInfo);
      switch (TGRI.FragLimit - maxScore)
      {
         case 3:
	 		if (fragsLeft > 3)
	 		{
	    		foreach AllActors(class 'PlayerPawn', PP)
	       		PP.ClientPlaySound(ThreeFragsLeftSound, False);
            fragsLeft = 3;
	 		}
         break;
         case 2:
	 		if (fragsLeft > 2)
         {
            foreach AllActors(class 'PlayerPawn', PP)
	       		PP.ClientPlaySound(TwoFragsLeftSound, False);
           	fragsLeft = 2;
         }
         break;
         case 1:
         if (fragsLeft > 1)
         {
            foreach AllActors(class 'PlayerPawn', PP)
               PP.ClientPlaySound(OneFragLeftSound, False);
            fragsLeft = 1;
         }
	 		break;
         default:
	 		break;
      }
   }
}

function ScoreKill(Pawn Killer, Pawn Other)
{
   local int i;

   i = 0;
   while (TRUE)
   {
      if (NextMutator != None)
	 		NextMutator.ScoreKill(Killer, Other);
      else break;
      if (i > 15)
         break;
      i++;
   }
   announce();

   if (NextMutator != None)
      NextMutator.ScoreKill(Killer, Other);
}

defaultproperties
{
   fragsLeft=4
   OneFragLeft=Sound'GenericScoreAnnounceSounds.OneFragLeft'
	TwoFragsLeft=Sound'GenericScoreAnnounceSounds.TwoFragsLeft'
	ThreeFragsLeft=Sound'GenericScoreAnnounceSounds.ThreeFragsLeft'
	TakenTheLead=Sound'GenericScoreAnnounceSounds.TakenTheLead'
	TiedForLead=Sound'GenericScoreAnnounceSounds.TiedForLead'
	LostTheLead=Sound'GenericScoreAnnounceSounds.LostTheLead'
}
